function clone (x, init)
	local r = {}
	for k, v in pairs( x ) do
		if type(v) == "table" then
			r[k] = clone (v)
		else  r[k] = v             
		end
	end
	if init then
		for k, v in pairs( init ) do
			r[k] = v
		end
	end
	setmetatable (r, getmetatable(x))
	return r
end


function copy (x)
	local r = {}
	for k, v in pairs( x ) do
		r[k] = v
	end
	setmetatable (r, getmetatable(x))
	return r
end


function show_table (table)
	for i, v in pairs( table ) do
		print (i, v)
	end
end


function dump_table (table, file)
	file:write ("{\n")
	for k, v in pairs( table ) do
		if type (v) == "number" then
			if type (k) == "string" then
				file:write (k.." = "..v..",\n")
			else
				file:write ("["..k.."] = "..v..",\n")
			end
		elseif type (v) == "string" then
			if type (k) == "string" then
				file:write (k.." = [["..v.."]],\n")
			else
				file:write ("["..k.."] = [["..v.."]],\n")
			end
		elseif type (v) == "table" then
			if type (k) == "string" then
				file:write (k.." = ")
			else	
				file:write ("["..k.."] = ")
			end
			dump_table (v, file)
			file:write (",\n")
		elseif type (v) == "boolean" then
			if v then
				file:write (k.." = true,\n")
			else
				file:write (k.." = false,\n")
			end
		end
	end
	file:write ("}")
end
PARSING_VERB_EXPECTED 	= 0
PARSING_UNKNOWN_WORD 	= 1
PARSING_UNDEFINED_IT 	= 2
PARSING_INVALID_AND		= 3
PARSING_OBJECT_MISSING	= 4
PARSING_UNDEFINED_ACTION = 5
PARSING_TOO_MANY_OBJECTS = 6
PARSING_UNDEFINED_AGAIN = 7
PARSING_GENERAL_ERROR = 8


function parse (text)
	text = string.gsub (text, ",", " and ")

	local state = 0
	local temp, verb, obj1, obj2, phrase
	local phrases = {}
	
	text = string.gsub (text, '%b""', function (s)
		table.insert (phrases, string.sub (s, 2, -2))
		return "#"
	end)
	
	for s in string.gfind (text, "%S+") do
		if state == 0 then
			obj1, obj2, phrase = nil, nil, nil
			verb = verbs:replace (s)
			if not verb then
				parsing_error (PARSING_VERB_EXPECTED, s)
				return
			end
			state = 1
		
		elseif state == 1 then
			temp = prepositions:replace (s)
			if temp then
				if not obj1 then
					obj1 = {prep = temp, nouns = {}}
				end
			else
				temp = nouns:replace (s)
				if temp then
					if not obj1 then
						obj1 = {nouns = {}}
					end
					table.insert (obj1.nouns, temp)
					state = 2
				else
					temp = special:replace (s)
					if temp == "it" then
						if not obj1 then
							obj1 = {}
						end
						if it then
							it.prep = obj1.prep
							obj1 = it
						else
							parsing_error (PARSING_UNDEFINED_IT)
							return
						end
						state = 2
					elseif temp == "then" or temp == "and" then
						post_parse (verb, obj1, obj2, phrase)
						state = 0
					
					elseif s == "#" then
						obj1 = nil
						phrase = phrases [1]
						table.remove (phrases, 1)
					
					else
						parsing_error (PARSING_UNKNOWN_WORD, s)
						return
					end
				end
			end
		
		elseif state == 2 then
			temp = prepositions:replace (s)
			if temp then
				obj2 = {prep = temp, nouns = {}}
				it = obj1
				state = 3
			else
				temp = nouns:replace (s)
				if temp then
					table.insert (obj1.nouns, temp)
				else
					temp = special:replace (s)
					if temp == "then" then
						post_parse (verb, obj1, obj2, phrase)
						it = obj1
						state = 0
					elseif temp == "and" then
						state = 4

					elseif s == "#" then
						if not obj1.nouns then
							obj1 = nil
						end
						phrase = phrases [1]
						table.remove (phrases, 1)
						
					else
						parsing_error (PARSING_UNKNOWN_WORD, s)
						return
					end
				end
			end
		
		elseif state == 3 then
			temp = nouns:replace (s)
			if temp then
				table.insert (obj2.nouns, temp)
			else
				temp = special:replace (s)
				if temp == "then" then
					post_parse (verb, obj1, obj2, phrase)
					it = obj1
					state = 0
				elseif temp == "it" then
					it.prep = obj2.prep
					obj2 = it
				elseif temp == "and" then
					state = 5
					
				elseif s == "#" then
					obj2 = nil
					phrase = phrases [1]
					table.remove (phrases, 1)
					
				else
					if prepositions:replace (s) then
					else
						parsing_error (PARSING_UNKNOWN_WORD, s)
						return
					end
				end
			end
		
		elseif state == 4 then
			temp = verbs:replace (s)
			if temp then
				post_parse (verb, obj1, obj2, phrase)
				it = obj1
				verb, obj1, obj2, phrase = temp
				state = 1
			else
				temp = nouns:replace (s)
				if temp then
					if not obj1.mult then
						obj1.mult = {}
					end
					table.insert (obj1.mult, {pos = table.getn (obj1.nouns), noun = temp})
					state = 2
				else
					temp = special:replace (s)
					if temp == "then" then
						
					else
						parsing_error (PARSING_INVALID_AND)
						return
					end
				end
			end
		
		elseif state == 5 then
			temp = verbs:replace (s)
			if temp then
				post_parse (verb, obj1, obj2, phrase)
				it = obj1
				verb, obj1, obj2, phrase = temp
				state = 1
			else
				temp = nouns:replace (s)
				if temp then
					if not obj2.mult then
						obj2.mult = {}
					end
					table.insert (obj2.mult, {pos = table.getn (obj2.nouns), noun = temp})
					state = 3
				else
					temp = special:replace (s)
					if temp == "then" then
						
					else
						parsing_error (PARSING_INVALID_AND)
						return
					end
				end
			end		
		
		
		elseif state == 6 then
		elseif state == 7 then
		end
	end
	
	it = obj1

	if obj2 and table.getn (obj2.nouns) == 0 then
		parsing_error (PARSING_GENERAL_ERROR)
		return
	end
	
	post_parse (verb, obj1, obj2, phrase)
end


function find_object (name)
	if table.getn (name) == 0 then
		return {player}
	end

	local next, objects = player.parent
	repeat
		objects = next:find (name)
		if next.parent then
			next = next.parent
		end
	until table.getn (objects) ~= 0 or next.parent ~= world
	
	if table.getn(objects) > 1 then
		for i, obj in ipairs(objects) do
			if obj:isme(name) then return {obj} end
		end
	end
	
	return objects
end


function post_parse (verb, obj1, obj2, phrase)
	if verb == "again" and not obj1 and not obj2 and not phrase then
		if not parsed_again then
			parsing_error (PARSING_UNDEFINED_AGAIN)			
			return
		end
		
		parsing_again = true
		post_parse (parsed_again[1], parsed_again[2], parsed_again[3], parsed_again[4])
		parsing_again = false
		return
	end
	if not parsing_again then
		parsed_again = {verb, obj1, obj2, phrase}
	end
	
	if obj1 and obj1.mult then
		local mult = obj1.mult
		obj1.mult = nil
		post_parse (verb, obj1, obj2, phrase)
		for i, p in ipairs (mult) do
			obj1.nouns [p.pos], p.noun = p.noun, obj1.nouns [p.pos]
			post_parse (verb, obj1, obj2, phrase)
		end
		obj1.mult = mult
		return
	end

	if obj2 and obj2.mult then
		local mult = obj2.mult
		obj2.mult = nil
		post_parse (verb, obj1, obj2, phrase)
		for i, p in ipairs (mult) do
			obj2.nouns [p.pos], p.noun = p.noun, obj2.nouns [p.pos]
			post_parse (verb, obj1, obj2, phrase)
		end
		obj2.mult = mult
		return
	end

	if obj1 then
		obj1.object = nil
		if obj2 then
			obj2.object = nil
		end
		local objects
		
		local all = false
		if obj1 and obj1.nouns [1] == "all" then
			all = true
		end
		
		objects = find_object (obj1.nouns)
		if table.getn (objects) == 1 then
			obj1.object = objects [1]
		
		elseif  all and table.getn (objects) > 0 then
			if obj2 and not obj2.object then
				local objects = find_object (obj2.nouns)
				if table.getn (objects) == 1 then
					obj2.object = objects [1]
				end
			end
			
			for i, obj in ipairs (objects) do
				obj1.object = obj
				do_parsed (verb, obj1, obj2, phrase)
			end
			return
			
		elseif not obj2 then
				local name1, name2 = clone (obj1.nouns), {}
				for i=1, table.getn (name1) do
					objects = find_object (name1)
					if table.getn (objects) == 1 then
						obj1.object = objects [1]
						obj2 = {nouns = clone (name2)}
						objects = find_object (name2)
						if table.getn (objects) == 1 then
							obj2 = {nouns = name2, object = objects [1]}
							obj1.nouns = name1
							break
						end
					end
					table.insert (name2, name1 [table.getn (name1)])
					table.remove (name1)
				end
		end
			
		if not obj1.object then
			objects = find_object (obj1.nouns)
			if table.getn(objects) == 0 then
				parsing_error (PARSING_OBJECT_MISSING, table.concat (obj1.nouns, " "))
			else
				parsing_error (PARSING_TOO_MANY_OBJECTS, objects)
			end
			return
		end
		
		if obj2 and not obj2.object then
			objects = find_object (obj2.nouns)
			if table.getn (objects) == 1 then
				obj2.object = objects [1]
			else
				obj2.objects = objects
			end
		end
	end
	
	do_parsed (verb, obj1, obj2, phrase)
end


function do_parsed (verb, obj1, obj2, phrase)
	local event, object = {prep = {}}, player
	event.verb = verb
	event.default = true
	if obj2 then
		event.prep [2] = obj2.prep
		event.name = obj2.nouns
		event.object = obj2.object
		event.objects = obj2.objects
	end
	
	if obj1 then
		event.prep [1] = obj1.prep
		event.default = nil
		object = obj1.object
	end
	
	if phrase then
		event.phrase = phrase
	end
	
	local next, hevent = object, copy (event)
	hevent.verb = verb
	hevent.target = object
	hevent.before = true
	repeat
		if next.hooks then
			for i=1,table.getn (next.hooks) do
				if not next.hooks [i] (hevent) then
					return
				end
			end
		end
		
		next = next.parent
	until not next
	
	if object [verb] then
		if type(object[verb]) == "function" then
			object [verb] (object, event)
		else
			print (object [verb])
		end
	else
		parsing_error (PARSING_UNDEFINED_ACTION, {verb, object.name})
		return
	end

	hevent.before = nil
	hevent.after = true
	next = object
	repeat
		if next.hooks then
			for i=1,table.getn (next.hooks) do
				next.hooks [i] (hevent)
			end
		end
		
		next = next.parent
	until not next
end


function parsing_error (err, detail)
	if err == PARSING_VERB_EXPECTED then
		print ("error: unknown verb '"..detail.."'")
	elseif err == PARSING_UNKNOWN_WORD then
		print ("error: unknown word '"..detail.."'")
	elseif err == PARSING_UNDEFINED_IT then
		print "error: undefined 'it'"
	elseif err == PARSING_INVALID_AND then
		print "error: invalid using of 'and'"
	elseif err == PARSING_OBJECT_MISSING then
		print ("error: couldn't find object '"..detail.."'")
	elseif err == PARSING_UNDEFINED_ACTION then
		print ("error: undefined action '"..detail [1].."' of object '"..detail [2].."'" )
	elseif err == PARSING_TOO_MANY_OBJECTS then
		print "error: too many objects on the scene:"
		for i, v in ipairs( detail ) do
			print (v.name)
		end
	elseif err == PARSING_UNDEFINED_AGAIN then
		print "error: undefined again"
	elseif err == PARSING_GENERAL_ERROR then
		print "error: syntax error"

	end
end

--=================================================================================================
vocabulary = {}


function vocabulary:add (template)
	if type (template) == "table" then		
		table.insert (self, template)
	else
		if string.sub (template, -1) ~= "\n" then
			template = template.."\n"
		end

		local templ 
		for s in string.gfind (template, "(.-)\n") do
			templ = {}
			for s in string.gfind (s, "%S+") do
				table.insert (templ, s)
			end
			self:add (templ)
		end
	end
end


function vocabulary:replace (word)
	local left, right
	for i, template in ipairs (self) do
		for i, t in ipairs (template) do
			left, right = false, false
			if string.sub (t, 1, 1) == "*" then
				left = true
				t = string.sub (t, 2)
			end
			if string.sub (t, -1) == "*" then
				right = true
				t = string.sub (t, 1, -2)
			end
			
			if left and right then
				if string.find (word, t) then
					return template[1]
				end
			elseif left then
				if t == string.sub (word, -string.len (t)) then
					return template[1]
				end
			elseif right then
				if t == string.sub (word, 1, string.len(t)) then
					return template[1]
				end
			else
				if word == t then
					return template[1]
				end
			end
		end
	end
end


function vocabulary:check ()
	for i, template in ipairs (self) do
		for j, t in ipairs (template) do
			t = string.gsub (t, "*", "")
			if t ~= self:checking_replace (t, i) then
				print ("error in word "..template[1]..": it replaces by "..self:checking_replace(t, i))
			end
		end
	end
end



function vocabulary:checking_replace (word, line)
	local left, right
	for i, template in ipairs (self) do
		if i ~= line then
			for i, t in ipairs (template) do
				left, right = false, false
				if string.sub (t, 1, 1) == "*" then
					left = true
					t = string.sub (t, 2)
				end
				if string.sub (t, -1) == "*" then
					right = true
					t = string.sub (t, 1, -2)
				end
				
				if left and right then
					if string.find (word, t) then
						return template[1]
					end
				elseif left then
					if t == string.sub (word, -string.len (t)) then
						return template[1]
					end
				elseif right then
					if t == string.sub (word, 1, string.len(t)) then
						return template[1]
					end
				else
					if word == t then
						return template[1]
					end
				end
			end
		end
	end
	return word
end


--=================================================================================================
verbs = clone (vocabulary)

-- vocabulary
verbs:add [[
again     
look  * 
inventory   inv  i
exit    goto
enter  
help   rules
startgame  start
quit bye   
stand   
clean tidy
up   
down  
climb    **
lie  
restart  
spusk    
think **
factos  * 
remember **
smoke *
smell *
open *
turnon *
read * *
elaborate  
east  
west  
north  
south  
southeast - 
northwest - 
northeast - 
southwest - 
exits  
let 
take  *
lisn **
katch * **
turnof *
repare **
speak * *
study   
pobeg *
zasun    
peek *
touch * **
spelles *  *
what 
brek * *
unlok   
parseron
parseroff
]]
-- /vocabulary

verbs:check ()
nouns = clone (vocabulary)

-- vocabulary
nouns:add [[
all 
room *
stol table desk bureau *
pol floor    
door *
my     
okno window     
myself      
first 
fork *
robot *
hare  *
kopf *
tabur *
instrum * *
sygn *
wall *
megafon *
patch  *
chulan *
potolok  *
telescop 
spichka *
kaban *
eyes *
mouth    
zub * *
kosh *
dust   
hand   
sink * *
nos *
skaf *
delat 
naverx * *
leaves *
surfac *
shelf * *
gnils * *
slit * 
dudka * *
]]
-- /vocabulary

nouns:check ()
prepositions = clone (vocabulary)

-- vocabulary
prepositions:add [[
in 
on at 
to 
into  
behind 
from 
with  
between 
about   
the an a
under 
around 
through  via 
up  
down 
]]
-- /vocabulary

prepositions:check ()
special = clone (vocabulary)

special:add [[
and 
then  
it her him            them 
]]

special:check ()
slib = {
	lang = { }
}

setmetatable(slib, {
	__index = function( self, key )
		return slib : get(key)
	end,
})

-- ============================================================================
-- collection to inherit

slib.collection = {
	__call = function(self, ...)
		if table.getn(self) == 0 then return end
		if self.rnd == true then
			local last = self.it
			if not last or table.getn(self) > 1 then
				self.it = last
				while self.it == last do
					self.it = math.random(table.getn(self))
				end
			else
				self.it = math.random(table.getn(self))
			end
		else
			self.it = self.it or 1
		end
		local text
		if table.getn(arg) > 0 then
			text = string.format(self[self.it], unpack(arg))
		else
			text = self[self.it]
		end
		if not self.rnd then
			self.it = math.mod(self.it, table.getn(self)) + 1
		end
		--print "\n"
		return text
	end,
}

-- ============================================================================
-- add new language library

function slib : add( language, library )
	assert(type(language)=="string" and type(library)=="string", "wrong arguments")
	library = string.gsub(library, "\\\n", "\r")
	
	local lib, it = { }
	string.gsub(library .. "\n", "(.-)\n", function( s )
		local green = false
		string.gsub(s, "%[([#]*)([%w.]+)%]", function( random, addr )
			it, green = lib, true
			string.gsub(addr, "%w+", function( name )
				if it[name] then
					it = it[name]
				else
					local coll = { rnd = (string.len(random) ~= 0) }
					setmetatable(coll, self.collection)
					it, it[name] = coll, coll
				end
			end)
		end)
		
		if not green and it then
			string.gsub(s, "%s*(%S.-)%s*$", function( phrase )
				phrase = string.gsub(phrase, "\r", "\n")
				table.insert(it, phrase)
			end)
		end
	end)
	
	self.lang[language] = lib
end

-- ============================================================================
-- set current language

function slib : set( language )
	assert(self.lang[language], "language " .. language .. " is missing in the library")
	self.current = language
end

-- ============================================================================
-- get top-level phrase by id (current language used)

function slib : get( id )
	assert(self.current, "the current language isn't set")
	local phrase = self.lang[self.current][id]
	assert(phrase, string.format("unknown phrase %q in %q ", id, self.current))
	return phrase
end

function slib : has( id )
	local phrase = self.lang[self.current][id]
	return phrase ~= nil
end

-- ============================================================================

local ru = [[

[hand.noput]
      .
,      .

[parser.doon]
<br><p align = "right"><a href = "parseron"> </a></p>

[parser.doof]
<br><p align = "right"><a href = "parseroff"> </a></p>

[inventory.empty]
     .
     .

[inventory.one]
   ,    :<br>

[inventory.many]
    :<br>

[status.start]
... ... ...

[look.default]
   .<br>
   ,   .<br>

[menucom.nule]
  

[menucom.one]
<br><a href = "look"><img src = "osm.gif"></img></a>&nbsp;&nbsp;&nbsp;&nbsp;<a href = "inventory"><img src = "inv.gif"></img></a>&nbsp;&nbsp;&nbsp;&nbsp;<a href = "factos"><img src = "mem.gif"></img></a>.

[status.one]
   .
 <a href= "look"></a>  .

[game.start]
<i>,   ...  ,      ,  ,  - .</i> <stop><clear><i>     , ,  -   ,          , -         ...</i> <stop><clear><i>       - ,  -     ;             . </i><stop><clear><i>   -,   ,    -...  ,      ... </i><stop><clear><i>        - ,   ,   ,      ...</i> <stop><clear><i>   ,      ,    ,       ,      - ...</i><stop><clear><i>... ...  - ,       - ,   .</i> <stop><clear>

[game.start2]
<u></u>:       ,  <u><b>""</u></b>. <br><u>:</u>       , <u><b>""</u></b> (),  <u><b>""</u></b> ().   -       .

[game.start3]
...   . <stop><clear>

[game.start4]
 ,     ,       ,   - ,   <a href = "look"></a>,     .<br><br>,    ?      ...<br><br>   ?

[escape.try.one]
,  ,     ,    .     ,     -...

[chulan.lookup.one]
  ,   , . , ,  <a href = "look okno"></a>,   .      <a href = "look shelf"></a>,    <a href = "look gnils"></a>,    ,   .

[chulan.climb.one]
     ,  . 

[chulan.walls.look1]
    .

[chulan.skaf.look1]
     ,        .  <a href = "look surfac skaf"></a>   ,        . 

[chulan.skaf.open1]
    . <stop>,   .<br>,     <a href = "look slit"></a>,  ,    - .
    ...<stop><br>,  .   .  <a href = "look slit"></a>   ,  , ,   .

[chulan.slit.look]
ٸ  , .

[chulan.skafdoors.look1]
    ,  . 


[chulan.skaf.closed]
<br><a href = "look door skaf"   .

[telescop.desc]
<br> <a href = "look telescop"><img src = "teleskop.gif"></img></a>     <a href = "look kosh"><img src = "koshike.gif"></img></a>.<br>
<br>  <a href = "look kosh"><img src = "koshike.gif"></img></a>  <a href = "look telescop"><img src = "teleskop.gif"></img></a>.<br>

[telescop.look1]
    .       <a href = "look hole"></a>,   <a href = "look wall"></a>.
      <a href = "look hole"></a>,  -   <a href = "look wall"></a>.

[koshik.look1]
  ,   - . 
     - .

[koshik.study1]
    .    - :        .
     .

[spichka.desc]
    .

[spichka.look1]
 , .

[dudka.look1]
     .

[spichka.caption]
<a href = "look spichka"><b> </b></a>
<a href = "look spichka"><b></b></a>

[dudka.caption]
<a href = "look dudka"><b></b></a>
<a href = "look dudka"><b></b></a>

[dudka.look1]


[chulan.look]
    , <a href = "look wall"><img src ="wall.gif"></img></a>&nbsp;&nbsp;  . - , ,     <a href = "look okno"><img src = "okna.gif"></img></a>.


[chulan.study]
 ,   .      - ,     . <br>   ,   ,      - ,      .

[sink.desc1]
    <a href = "look sink"></a>.<br>
   <a href = "look sink"></a>   .<br>
   <a href = "look skaf"></a>  <a href = "look sink"></a>.<br>

[sink.look1]
 ,   . <br>,    . <br><br>,         .<br>
    . ,  . <br><br> ,   ,      ?<br>

[sink.firstlook]
    ...<stop><br>

[sink.study]
  -     . <br>

[sink.leaves]
 ,   -  ,   <a href = "look leaves"></a>.
     <a href = "look leaves"></a>.    ?

[sink.bugs]
 ,  ,         - ...
      .

[noise.chulan.woods]
,  -    ...
-  ,   ,     ...
-    ;   ,   ...

[noise.chulan.bugs.falling]
  ,  -    ...
 - .   ,   ,       ,   .
    -    ...

[kaban.desc1]
 ,   <a href = "look skaf"><img src = "skafa.gif"></img></a>, ...  - <a href = "look kopf"><img src = "kopf.gif"></img></a>.<br>
 ,    <a href = "look skaf"><img src = "skafa.gif"></img></a>,  - <a href = "look kopf"><img src = "kopf.gif"></img></a>.<br>
- <a href = "look kopf"><img src = "kopf.gif"></img></a>     <a href = "look skaf"><img src = "skafa.gif"></img></a>.<br>

[kaban.desc2]
     .<br>
     .<br>

[kaban.look1]
,   - .       .    .
     .

[kaban.look2]
  ,    ,     .
   .       .

[kaban.study1]
       ? <stop><clear>  ,   ?  !   . <br>    .
      ? <stop><clear>   .  ,      ;  ,    . <br>  ,    -   .

[ceiling.study]
  : ,      - ,      .

[player.lookat]
   ,  ,      .
  ,    ,        :     -       .

[golova.pres]
-    . 

[tumba.pres]
    ,          . 

[teleskop.desc1]
 -    ,   -    . 
 ,    ,  ,     .

[dirkes.desc1]
    ,     .
-           - .     ?

[chulan.noise]
<i>-    ...</i>
<i>    ...</i>
<i> ,  -  .</i>

[golova.desc1]
   .  ,     .       -  .

[kolda.bulda]
   .

[room.look.invalid]
  %s,   .

[button.further]
<p align = "right"><a href = "look">&raquo;&raquo;</a></p>

[prop.gendesc]
  %s .

]]

slib : add( "ru", ru )
slib : set "ru"
--======================================================================================
function move_node (obj, target)
	if obj.parent then
		remove_node (obj)
	end
	obj.parent = target
	if not target.items then
		target.items = {}
	end
	table.insert (target.items, obj)
end


function remove_node (obj)
	local items = obj.parent.items
	for i, item in ipairs(items) do
		if items [i] == obj then
			table.remove (items, i)
			break
		end
	end
	
	if obj.parents then
		local ps
		ps, obj.parents = obj.parents, nil
		while table.getn (ps) ~= 0 do
			obj.parent = ps [table.getn(ps)]
			table.remove (ps)
			remove_node (obj)
		end
	end
end


function show_tree (node, level)
	if not node then
		node, level = world, 0
	end
	
	print (string.rep ("\t", level)..node.uid)
	
	if not node.items then
		return
	else	
		for i, v in ipairs (node.items) do
			show_tree (v, level + 1)
		end
	end
end


function show_parents (node)
	print (node.uid)
	if node.parent then
		show_parents (node.parent)
	end
end
--======================================================================================
ifobject = {}

function ifobject:add (...)
	if not self.items then
		self.items = {}
	end

	if type (arg [1]) == "table" then
		for i, v in ipairs (arg) do
			if not v.parent  then
				move_node (v, self)
			else
				table.insert (self.items, v)
				if not v.parents then
					v.parents = {}
				end
				table.insert (v.parents, self)
			end
		end
	else
		if not self.hooks then
			self.hooks = {}
		end
		table.insert (self.hooks, arg [1])
	end
end


function ifobject:moveto (target)
	move_node (self, target)
end


function ifobject:remove (object)
	if not object then
		if self.parent then
			remove_node (self)
		end
	else
		for i, v in ipairs( self.items ) do
			if v == object then
				table.remove (self.items, i)
				return
			end
		end
	end
end


function ifobject:show ()
	show_tree (self, 0)
end

local ifobject_meta = {
	__index = function( self, key )
		local result
		if self.data.parent then
			local parent = _G[self.data.parent]
			if parent and parent ~= self and key ~= "items" then
				result = parent[key]
			end
		end
		if type(result) == "nil" then
			result = self.data[key]
		end		
		return result
	end,
	__newindex = function( self, key, value )
		if type(self.data[key]) ~= "nil" then
			self.data[key] = value
		else
			rawset(self, key, value)
		end
	end
}

function create_ifobject (uid, name)
	local obj = clone (ifobject, {["uid"] = uid, ["name"] = {}, data = {}, caption = ""})
	setmetatable( obj, ifobject_meta )
	getfenv(0)[uid] = obj
	obj.name = name	
	return obj
end


function ifobject:create (uid, name)
	self:add (create_ifobject (uid, name))
end


function ifobject:find (name, list)	
	if not list then list = {} end

	if string.len (self.name) ~= 0 then
		local addthis = true
		for i, noun in ipairs (name) do
			if not string.find (self.name, noun) and noun ~= "all" then
				addthis = false
				break
			end
		end
		
		if table.getn (name) == 1 and name [1] == "all" then
			addthis = false
		end
		
		if addthis then
			table.insert (list, self)
		end
	end
	
	if self.items then
		for i, item in ipairs (self.items) do
			item:find (name, list)
		end
	end
	
	return list
end

function ifobject:isme( name )
	local rest = self.name
	for i, n in ipairs(name) do rest = string.gsub( rest, n, "" ) end
	return not string.find( rest, "%S" )
end


function ifobject:contains (object)
	if self == object then
		return true
	end
	
	if self.items then
		for i, item in ipairs (self.items) do
			if item:contains (object) then
				return true
			end
		end
	end
	
	return false
end


function ifobject:clone (object, uid)
	local temp1, temp2, temp3
	object.items, object.parent, object.parents, temp1, temp2, temp3 = {}, nil, nil, object.items, object.parent, object.parents
	local result = clone (object)
	object.items, object.parent, object.parents = temp1, temp2, temp3
	result.parent = nil
	result.uid = uid
	getfenv(0) [uid] = result
	result:moveto (self)
end

function ifobject:check (parent, cookie)
	cookie = cookie or { }
	if self ~= _G[self.uid] then
		print ("  "..self.uid.."  "..parent.uid)
		parent:remove (self)
		return
	end

	if cookie[self] then
		print (self.uid.."    "..parent.uid..".  ...")
		parent:remove (self)
		return
	end
	cookie[self] = true
	if self.items then
		for i, v in ipairs( self.items ) do
			v:check (self, cookie)
		end
	end
	cookie[self] = nil
end


function ifobject:belongto (object)
	if self.parent then
		local next = self.parent
		while next do
			if next == object then
				return true
			end
			next = next.parent
		end
	else
		return false
	end
	return false
end


function ifobject:count ()
	if not self.items then
		return 0
	end

	return table.getn (self.items)
end


--======================================================================================
function start ()
	while true do
		io.output():write ('> ')
		parse (io.input():read())
		print ("")
	end
end


function save_node (node, file)
	if node.data then
		file:write (node.uid..".data = ")
		dump_table (node.data, file)
		file:write ("\n")
	end
	
	file:write (node.uid..".name = '"..node.name.."'\n")
	if (node.parent) then
		file:write (node.uid..":moveto ("..node.parent.uid..")\n")
	end
	
	if node.items then
		for i, v in ipairs( node.items ) do
			save_node (node.items [i], file)
		end
	end
end


function save (filename)
	local file = io.open (filename, "w")
	save_node (world, file)
	file:close()
end


function free_node (node)
	while node.items and table.getn (node.items) ~= 0 do
		free_node (node.items [1])
	end
	
	node:remove ()
end


function load (filename)
	free_node (world)

	dofile (filename)
end

--======================================================================================
PARSE_ERROR = {[[
      .]], [[
    .]], [[
,     .]], [[
   "".]], [[
    .]], [[
    .]], [[
     .  .]], [[
,      .]], [[
 - .]]}

sixdays_print_error = print

function parsing_error (error)
	local pe = PARSE_ERROR [error + 1]
	local temp, n = string.gsub (pe, "[^\n\r]+", " ")
	n = math.random (n)
	temp = 1
	string.gsub (pe, "[^\n\r]+", function (s)
		if temp == n then
			sixdays_print_error (s)
		end
		temp = temp + 1
	end)
end
--======================================================================================

create_ifobject ("world", "")
world:create ("player", "")
world.items = {}
function world:flags (event)
-- OBJECT MUST BE MIRROR.TRUE if we want to "look at me", and do LOOKYMIRROR as event

-- sink.data.leaves_in = true

-- sink.data.bugs_in = true

--    -     

-- x:pln (slib.noise.bugs.falling ())
-- x:pln (slib.noise.woods())

-- ,       .

--     ,      :
--   -      
--           

--    

--    ,  "   !"  ,     
--    ,      ,     ,

end
function world:s_up (event)
-- x:pln "$(newstatus ( slib.status.start() )) $(slib.game.start()) <stop><clear>$(newstatus ( slib.status.start() )) $(slib.game.start2())<stop><clear>$(newstatus ( slib.status.one() )) $(slib.game.start3())"	

-- world.data.input = "<tr><td>$(input)"
world.data.input = ""


-- x:p "$(newstatus (slib.status.start()))"
-- x:pln (slib.game.start())
-- x:pln (slib.game.start3())

sink.data.leaves_in = true
x:p "$(newstatus (slib.status.one()))"
x:p "$(newmenucom (slib.menucom.one()))"

x:pln (slib.game.start4())

end
function world:constructor ()
math.randomseed( os.time() )

x:fgcolor "#571B7e"
x:bgcolor "#E3E4FA"


--   
x:body [[
<table width=100%>
<tr bgcolor=$(fgcolor)><td><i><font color=$(bgcolor)>$(player.status)</font></i></align>
<tr><td>$(output)
$(world.input)
<tr><td>$(g_error)
<tr><td><i>$(player.noises)</i>
<tr><td><hr>
<tr><td><smaller>$(player.menucom)</smaller>
<tr><td><smaller>$(player.parser_opt)</smaller>

</table>	
]]

--   ----
x:on_input( function( text )
	g_error = ""; x:clear(); player.noises = ""; parse(text)
	if #g_error > 0 then x:restore() end
end )

--   
sixdays_print_error = function( text ) g_error = text end
g_error = ""

x:anykey "<table width=100% cellspacing=0 cellpadding=0><tr><td align=right><table><tr><td bgcolor=$(fgcolor)><font color=$(bgcolor)>&raquo;&raquo;&raquo; -- </font></table></table>"

--   
x.tags.inv = "<b>"
x.tags.mem = "<font color=#571B7e><i>"
x.tags.ob = "<font color=#571B7e>"
x.tags.sp = "<i><b>"
x.tags.bigger = "<font size=$(fontsize+1)>"
x.tags.smaller = "<font size=$(fontsize-1)>"
x.tags.nz = "<font color=#737CA1>"


--  
newstatus = function(status) player.status = status end
newnoises = function(noises) player.noises = noises end
newmenucom = function(menucom) player.menucom = menucom end

world:s_up (event)

end

world.data = {
input = [[<tr><td>$(input)]],
}world:create ('chulan', 'chulan')
chulan:add (function (event)
local disallowed = { knock = true, open = true, read = true, east = true, west = true, north = true, south = true, southwest = true, northwest = true, southeast = true, southwest = true, exit = true, enter = true }
if event.after and not disallowed[event.verb] then

	chulan:noiz_check (event)

return false
end
return true
end)
function chulan:noiz_check (event)
self.noizcount = self.noizcount - 1
	player.noises = ""

if self.noizcount <= 0  then 

	if not self.noiz_shot then

		if sink.bugs_in then
		player.noises = slib.noise.chulan.bugs.falling()
		
		else 
		player.noises = slib.noise.chulan.woods()
		end
	self.data.noiz_shot = true


	self.noizcount = math.random(3) + 1

	else

	self.noiz_shot = false
	
end
end
end
function chulan:study (event)
x:pln (slib.chulan.study () )
end
function chulan:look (event)
x:pln ( slib.chulan.look ())
x:p "$(newmenucom (slib.menucom.one()))"
self:menulok (event)
end
function chulan:menulok (event)
x:p "$(newmenucom (slib.menucom.one()))"
end
chulan.data = {
noizcount = 0,
}chulan:create ('player', 'myself')
function player:exit (event)

end
function player:pobeg (event)
x:pln (slib.escape.try.one ())
end
function player:factos (event)

end
function player:study (event)
if event.target ~= player then
self.parent:study (event)
	
	else 
		if event.object.mirror then
		event.object:lookymirror (event)	
		else
		x:pln (slib.player.lookat ())
		end

end
end
function player:save (event)
save "savegame.lua"
end
function player:checkparser (event)
if player.parser_status.active then
	
	
else 



end
end
function player:think (event)

end
function player:chekinv (event)
if karman:count () == 0 then
	return slib.inventory.empty()
end

if karman:count() == 1 then
	local inventory = slib.inventory.one()

	local uid = karman.items[ 1 ].uid
	inventory = inventory..slib[uid].caption()..",  "

	inventory = string.sub( inventory, 1, -3 ).."."

	return inventory

end


local inventory = slib.inventory.many()
for i = 1, karman:count() do
	local uid = karman.items[ i ].uid
	inventory = inventory..slib[uid].caption()..",  "
end

inventory = string.sub( inventory, 1, -3 ).."."

return inventory
end
function player:look (event)
if event.target ~= player then

x:pln (slib.look.default())

player.parent:look (event)
-- x:pln "<br>"
for i = 1, player.parent:count() do
local item = player.parent.items[i]
if item.prop and item.parent ~= player then item:dodesc() 
end
end
	
	else 
		if event.object.mirror then
		event.object:lookymirror (event)	
		else
		x:pln (slib.player.lookat ())
		end
end
end
function player:lie (event)

end
function player:chekstuff (event)
for i = 1, player.parent:count() do
local item = player.parent.items[i]
if item.prop and item.parent ~= player then item:dodesc() 
end
end
end
function player:restart (event)

end
function player:inventory (event)
x:p "$(newmenucom (slib.menucom.nule()))"
x:pln "$(player:chekinv())"

x:pln (slib.button.further())



end
function player:load (event)
load "savegame.lua"
end
function player:up (event)
player.parent:up (event)
end
function player:spelles (event)
for i = 1, spells:count() do
local item = spells.items[i]
if item.activ then item:dodesc() 
end
end
end
function player:remember (event)

end
function player:parseron (event)
world.data.input = "<tr><td>$(input)"
player.parser_opt = '$(slib.parser.doof())'
end
function player:parseroff (event)
world.data.input = ""
player.parser_opt = '$(slib.parser.doon())'
end
function player:help (event)

end
function player:down (event)

end
function player:menulok (event)

end
player.data = {
parser_status = [[active]],
parser_opt = [[$(slib.parser.doon())]],
menucom = [[]],
}player:create ('karman', 'karman')
karman.data = {
}karman:create ('dudka', 'dudka')
function dudka:look (event)
x:p (slib.dudka.look1())
end
function dudka:menuspek (event)

end
dudka.data = {
}karman:create ('spichka', 'spichka')
function spichka:look (event)
x:p (slib.spichka.look1())
end
function spichka:dodesc (event)
x:pln (slib.spichka.desc ())
end
function spichka:menuspek (event)

end
spichka.data = {
prop = true,
}player:create ('hand', 'hand')
function hand:zasun (event)
if event.object == mouth then

x:pln "rukaban"
	
	else
		
x:pln (slib.hand.noput ())
end
end
function hand:menuspek (event)

end
hand.data = {
}player:create ('nos', 'nos')
function nos:zasun (event)

end
function nos:menuspek (event)

end
function nos:study (event)

end
function nos:look (event)

end
nos.data = {
}player:create ('spells', '')
function spells:menuspek (event)

end
spells.data = {
}spells:create ('xop', 'xop')
function xop:dodesc (event)

end
xop.data = {
}spells:create ('gyr', 'gyr')
function gyr:dodesc (event)

end
gyr.data = {
}spells:create ('lur', 'lur')
lur.data = {
}spells:create ('urp', 'urp')
function urp:dodesc (event)

end
urp.data = {
}chulan:create ('potolok', 'potolok')
function potolok:peek (event)

end
function potolok:climb (event)

end
function potolok:stand (event)

end
function potolok:look (event)

end
function potolok:study (event)

end
function potolok:menuspek (event)

end
potolok.data = {
}chulan:create ('telescop', 'telescop')
function telescop:look (event)

end
function telescop:dodesc (event)
x:pln (slib.telescop.desc ())
end
function telescop:menuspek (event)

end
telescop.data = {
prop = true,
}chulan:create ('kabanh', 'kopf kaban')
function kabanh:talk (event)

end
function kabanh:study (event)

end
function kabanh:menuspek (event)

end
function kabanh:take (event)

end
function kabanh:look (event)
if self.checked then

x:pln (slib.kaban.look2())
	
	else
		x:pln (slib.kaban.look1())
end


end
kabanh:add (function (event)

if event.before then
	local stuzer = kabanh
	if event.target ~= stuzer then
		event.prep[1], event.prep[2] = event.prep[1] or "", event.prep[2] or ""
		local cmd = event.verb .. " " .. event.prep[1] .. " " .. stuzer.name .. " "
		cmd = cmd .. " " .. event.prep[2] .. " " .. table.concat(event.names or {}, " ")
		parse( cmd )
		return false
	end
end

return true
end)
function kabanh:dodesc (event)
if self.checked then
x:pln (slib.kaban.desc2 ())
	
else 	

x:pln (slib.kaban.desc1 ())

end
end
kabanh.data = {
prop = true,
}kabanh:create ('eyes', 'eyes')
eyes.data = {
}kabanh:create ('mouth', 'mouth')
mouth.data = {
}kabanh:create ('zub', 'zub')
zub.data = {
}chulan:create ('hole', 'hole')
function hole:peek (event)

end
function hole:menuspek (event)

end
function hole:study (event)

end
function hole:look (event)

end
hole.data = {
}chulan:create ('wall', 'wall')
function wall:study (event)

end
function wall:look (event)
x:p (slib.chulan.walls.look1())
end
function wall:menuspek (event)

end
wall.data = {
}chulan:create ('kosh', 'kosh')
function kosh:climb (event)

end
function kosh:study (event)

end
function kosh:look (event)

end
function kosh:open (event)

end
function kosh:menuspek (event)

end
kosh.data = {
}kosh:create ('dust', 'dust')
dust.data = {
}chulan:create ('sink', 'sink')
function sink:peek (event)
self:study (event)
end
function sink:stand (event)

end
function sink:menuspek (event)

end
function sink:take (event)

end
function sink:look (event)
x:pln (slib.sink.look1 ())
if self.checked  then
	if sink.data.leaves_in then
	x:pln (slib.sink.leaves ())
	
	end
	
	if sink.data.bugs_in then
	
	x:pln (slib.sink.bugs ())
		
	end
		
			
else 
end
end
function sink:study (event)
if not self.checked then
x:pln (slib.sink.firstlook ())
	
	if sink.data.leaves_in then
	x:pln (slib.sink.leaves ())
	
	end
	
	if sink.data.bugs_in then
	
	x:pln (slib.sink.bugs ())
		
	end

sink.data.checked = true

else 

x:pln (slib.sink.study ())
	if sink.data.leaves_in then
	x:pln (slib.sink.leaves ())
	
	end
	
	if sink.data.bugs_in then
	
	x:pln (slib.sink.bugs ())
		
	end



end
end
function sink:dodesc (event)
x:pln (slib.sink.desc1 ())
end
sink.data = {
prop = true,
}sink:create ('leaves', 'leaves')
function leaves:study (event)

end
function leaves:look (event)

end
function leaves:take (event)

end
leaves.data = {
}chulan:create ('skaf', 'skaf')
function skaf:dodesc (event)

end
function skaf:study (event)

end
function skaf:look (event)
x:p (slib.chulan.skaf.look1())
if skaf.closed then
x:p (slib.chulan.skaf.closed())
	else
x:p (slib.chulan.skaf.open1())
end
end
function skaf:open (event)
x:p (slib.chulan.skaf.open1 () )
end
function skaf:menuspek (event)

end
skaf.data = {
prop = true,
}skaf:create ('surfac', 'surfac skaf')
function surfac:study (event)

end
function surfac:look (event)

end
function surfac:touch (event)

end
surfac.data = {
}skaf:create ('doors', 'door skaf')
function doors:brek (event)

end
function doors:look (event)
x:p (slib.chulan.skafdoors.look1())

end
function doors:open (event)
skaf:open (event)
end
doors.data = {
}skaf:create ('slit', 'slit')
function slit:study (event)

end
function slit:look (event)

end
slit.data = {
}chulan:create ('naverx', 'naverx')
function naverx:climb (event)

end
function naverx:study (event)

end
function naverx:look (event)

end
function naverx:menuspek (event)

end
function naverx:go (event)

end
naverx.data = {
}chulan:create ('okna', 'okno')
function okna:brek (event)

end
function okna:climb (event)

end
function okna:look (event)

end
function okna:menuspek (event)

end
okna.data = {
}chulan:create ('uplook', 'naverx')
function uplook:look (event)
x:p (slib.chulan.lookup.one())
end
function uplook:climb (event)
x:p (slib.chulan.climb.one())
end
uplook.data = {
}chulan:create ('shelf', 'shelf')
function shelf:look (event)

end
function shelf:menuspek (event)

end
shelf.data = {
}shelf:create ('gnils', 'gnils')
function gnils:take (event)

end
function gnils:look (event)

end
gnils.data = {
}world:create ('props', '')
props.data = {
}props:create ('klopofon', 'klopofon')
function klopofon:dodesc (event)
x:pln (slib.prop.gendesc (self.name))
end
klopofon.data = {
prop = true,
}props:create ('head', 'golova')
function head:look (event)
self.parent:look (event)
end
head.data = {
}world:constructor ()
PARSE_ERROR = {[[      .]],[[    .]],[[,     .]],[[   "".]],[[ -    .
  . ,  ?]],[[,  .
  -  .
, , -   .]],[[     .  .]],[[,      .]],[[ - .]],}